--------------------------------------------------------------------------
-- LibEvent.lua 
--------------------------------------------------------------------------
--[[
LibEvent
- Tiny Event Lib

author: AnduinLothar    <karlkfi@cosmosui.org>

Change List
v1.0
- Hopefully one version is enough!

$Date: 2007-07-03 21:22:03 -0500 (Tue, 03 Jul 2007) $
$Rev: 5009 $

]]--

local lib, oldminor = LibStub:NewLibrary("LibEvent", 1)
if not lib then return end

------------------------------------------------------------------------------
--[[ Example Usage ]]--
------------------------------------------------------------------------------

--[[

local eventTable = {}

function MyAddOnTable.events.PLAYER_ENTERING_WORLD(eventFrame, event, ...)
	-- event == "PLAYER_ENTERING_WORLD"
	DoStuff()
	-- 'Unregister'
	MyAddOnTable.events.PLAYER_ENTERING_WORLD = nil
end

local LibEvent = LibStub("LibEvent")

-- Create global frame MyAddOnFrame to handle the events
local eventFrame = LibEvent.RegisterFrameEvents("MyAddOnFrame", MyAddOnTable.events)
-- or -- Create a no-name frame to handle the events
local eventFrame = LibEvent.RegisterFrameEvents(nil, MyAddOnTable.events)
-- or -- Use LibEvent's frame 
-- WARNING: Can't unregister and has a bit more overhead, so don't use for frequently called events
LibEvent:RegisterFrameEvents(MyAddOnTable.events)


--optionally store the event table for modification later
MyAddOnTable = {}
MyAddOnTable.events = eventTable

]]--


------------------------------------------------------------------------------
--[[ Public Functions ]]--
------------------------------------------------------------------------------

-- Event Registration
-- eventTable[event_name] = onEventFunction
function lib.RegisterEvents(frame, eventTable)
	
	-- Create a frame if none exist by that name
	if (type(frame) == "string") then
		if (not _G[frame]) then
			frame = CreateFrame("Frame", frame)
		else
			frame = _G[frame]
		end
	elseif (frame == lib) then
		if (not lib.frame) then
			lib.frame = CreateFrame("Frame")
		end
		frame = lib.frame
	elseif (frame == nil) then
		frame = CreateFrame("Frame")
	end
	
	for event, func in pairs(eventTable) do
		frame:RegisterEvent(event)
	end
	
	local SetScript = "SetScript"
	if frame:GetScript("OnEvent") then SetScript = "HookScript" end
	
	-- Pass a wrapper so that the event handler can modified after registration
	frame[SetScript](frame, "OnEvent", function(eventFrame, event, ...)
		local func = eventTable[event]
		if func then func(eventFrame, event, ...) end
	end)
	
	-- Don't return the frame if it's LibEvent's
	-- We don't want anyone unregistering events used by other addons
	if (frame ~= lib.frame) then
		return frame
	end
end

